# -*- coding: utf-8 -*-

import os, re
from .cache_utils import cpickle_cache
from ._current_dir import current_dir
from .design_pattern import singleton ,cached_property

@singleton()
class LazyData(object):

    @cached_property
    def en_us_dict(self):
        """
        werid example.
        >>> en_us_dict.check(u"跑")
        True
        """
        # `pip install pyenchant`
        import enchant
        return enchant.Dict("en_US")

    @cached_property
    def two_length_words(self):
        """
        检查 2个字符 的字符组 是否是正常单词 或 词组
        e.g. `"At" in two_length_words`
        """
        return self.nltk_abc_data[0]

    @cached_property
    def regular_words(self): return self.nltk_abc_data[1]

    @cached_property
    def nltk_abc_data(self):
        nltk = self.nltk_download('abc')
        def load__nltk_abc_data__func():
            # `pip install marisa_trie`
            import marisa_trie
            from .regexp_utils import regexp
            words = nltk.corpus.abc.words()

            two_length_words_data = marisa_trie.Trie( \
                        sorted( \
                        list( \
                        set(\
                            [w1 for w1 in words \
                                if (len(w1) == 2) and \
                                    re.compile("[a-z]", re.IGNORECASE).match(w1[0]) \
                            ] \
                        ))))

            regular_words_data = marisa_trie.Trie([ \
                    unicode(w1) for w1 in words if regexp.word.match(w1)])

            return [two_length_words_data, regular_words_data]

        return cpickle_cache(current_dir + "/nltk_abc.cPickle", \
                load__nltk_abc_data__func, True)


    @cached_property
    def lemmatize(self):
        """
        return lemmatize method.

        必须得注明 词形 才行
        >>> lmtzr.lemmatize("having")
        'having'
        >>> lmtzr.lemmatize("having", 'v')
        'have'

        http://stackoverflow.com/questions/771918/how-do-i-do-word-stemming-or-lemmatization
        """
        nltk = self.nltk_download('wordnet')
        lmtzr = nltk.stem.wordnet.WordNetLemmatizer()

        def func(word1, tag1=None):
            if tag1 is None:
                word1 = lmtzr.lemmatize(word1, 'v')
                tag1 = 'n'
            return lmtzr.lemmatize(word1, tag1)
        return func

    @cached_property
    def tagged_words__dict(self):
        nltk = self.nltk_download('brown')
        return {w1:t1.upper() for w1, t1 in nltk.corpus.brown.tagged_words()}

    def nltk_download(self, package):
        # `pip install nltk`
        import nltk
        if nltk.data.path[0] != current_dir:
            nltk.data.path.insert(0, current_dir)

        if not os.path.isdir(os.path.join(current_dir, 'corpora', package)):
            nltk.download(info_or_id=package, download_dir=current_dir)
        return nltk

    @cached_property
    def jieba(self):
        import jieba
        return jieba


ld = LazyData()



# -*- coding: utf-8 -*-

from collections import Counter
from .design_pattern import singleton

@singleton()
class ListUtilsClass(object):

    def most_common_inspect(self, list1):
        new_list = []
        for s1 in list1:
            if not isinstance(s1, unicode):
                s1 = str(s1).decode("UTF-8")
            new_list.append(s1)

        cc = Counter(new_list).most_common()

        if len(cc) > 0:
            max_len = len(max([c1[0] for c1 in cc], key=lambda x1: len(x1))) + 5

            for c1 in cc:
                print c1[0].ljust(max_len, ' '), ' : ', c1[1]

        return cc


    def uniq_seqs(self, seqs, uniq_lambda=None):
        if uniq_lambda is None: return list(set(seqs))

        __uniq = set([])
        __remove_idxes = []

        for idx1, seq1 in enumerate(seqs[:]):
           __id = uniq_lambda(seq1)
           if __id in __uniq:
               __remove_idxes.append(idx1)
           else:
               __uniq.add(__id)

        new_seqs = []
        for idx1, seq1 in enumerate(seqs[:]):
            if idx1 not in __remove_idxes:
                new_seqs.append(seq1)

        seqs = new_seqs

        return seqs

ListUtils = ListUtilsClass()
uniq_seqs = ListUtils.uniq_seqs



# -*- coding: utf-8 -*-

class ItemIncrementIdDict(object):
    """ Assign an auto increment integer to item, e.g. an object_id.

    Usage:

    ItemIncrementIdDict.fetch(obj1) # => 1
    ItemIncrementIdDict.fetch(obj1) # => 1
    ItemIncrementIdDict.fetch(obj2) # => 2
    ItemIncrementIdDict.fetch(obj5) # => 3
    ItemIncrementIdDict.fetch(obj2) # => 2
    """

    count = 0
    object_id_to_increment_dict = dict()

    @classmethod
    def fetch(cls, object_id1):
        if cls.object_id_to_increment_dict.has_key(object_id1):
            return cls.object_id_to_increment_dict[object_id1]
        else:
            cls.count += 1
            cls.object_id_to_increment_dict[object_id1] = cls.count
            return cls.count



# -*- coding: utf-8 -*-

import re
from .lazy_data import ld

def is_regular_word(str1):
    if isinstance(str1, unicode): str1 = str1.encode("UTF-8")
    str1 = str(str1)

    return (str1 == 'a') or (str1 == 'I') or ((len(str1) >= 2) and ld.en_us_dict.check(str1))

def jieba_parse(str1):
    """ 精确模式 """
    return list(ld.jieba.cut(str1, cut_all=False))



# -*- coding: utf-8 -*-

def set_default_value(lambdas, msg=u""):
    val = None
    for lambda1 in lambdas:
        if val is None:
            try:
                val = lambda1()
            except:
                continue
    assert val is not None, msg
    return val



# -*- coding: utf-8 -*-

from setuptools import setup

setup(
    name='etl_utils',
    version='0.1.4',
    url='http://github.com/17zuoye/etl_utils/',
    license='MIT',
    author='David Chen',
    author_email=''.join(reversed("moc.liamg@emojvm")),
    description='etl utils',
    long_description='etl utils',
    packages=['etl_utils'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'werkzeug',
        'progressbar == 2.2', # 和目前 2.3 版本 API 更改了 20141103
        'termcolor',
        'pysingleton',
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



# -*- coding: utf-8 -*-

import logging
from .string_utils import StringUtils

class BufferLogger(object):
    """日志缓存管理

    使用说明:
    append, inspect, clear

    临时打印:
    p
    """

    def __init__(self, logger_file_name):
        self.__init_logger__(logger_file_name)
        self.buffer1 = list()

    def append(self, *strs):
        self.buffer1.append(StringUtils.merge(*strs))
    info = append

    def inspect(self):
        buffer_copy = self.buffer1[:]
        self.buffer1 = list()
        for b1 in buffer_copy:
            self.logger.info(b1)

    def clear(self):
        self.buffer1 = list()

    def p(self, *strs):
        print StringUtils.merge(*strs)

    def __init_logger__(self, logger_file_name):
        # copied from http://kenby.iteye.com/blog/1162698
        # 创建一个logger
        self.logger = logging.getLogger()

        # 创建一个handler，用于写入日志文件
        fh = logging.FileHandler(logger_file_name)
        fh.setLevel(logging.DEBUG)

        # 再创建一个handler，用于输出到控制台
        ch = logging.StreamHandler()

        # 定义handler的输出格式
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)

        # 给logger添加handler
        self.logger.addHandler(fh)
        self.logger.addHandler(ch)
